@@ -28,6 +28,7 @@ from paramtools.schema import (
 from paramtools.select import (
     select,
     select_eq,
+    select_ne,
     select_gt,
     select_gte,
     select_gt_ix,
@@ -46,11 +47,12 @@ from paramtools.utils import (
     hashable_value_object,
     filter_labels,
     make_label_str,
+    SortedKeyList,
 )
 
 
 name = "paramtools"
-__version__ = "0.13.1"
+__version__ = "0.14.1"
 
 __all__ = [
     "SchemaFactory",
@@ -94,5 +96,6 @@ __all__ = [
     "hashable_value_object",
     "filter_labels",
     "make_label_str",
+    "SortedKeyList",
     "ValueObject",
 ]
