@@ -40,6 +40,7 @@ collision_list = [
     "_errors",
     "_warnings",
     "select_eq",
+    "select_ne",
     "select_gt",
     "select_gte",
     "select_lt",
@@ -71,6 +72,7 @@ collision_list = [
     "errors",
     "warnings",
     "from_array",
+    "parse_labels",
     "read_params",
     "set_state",
     "specification",
