@@ -219,6 +219,18 @@ class TestSchema:
             "uses_extend_func": False,
         }
 
+        class Params3(Parameters):
+            array_first = True
+            label_to_extend = "hello"
+            defaults = {"schema": {"operators": {"array_first": True}}}
+
+        params = Params3(array_first=False, label_to_extend=None)
+        assert params.operators == {
+            "array_first": False,
+            "label_to_extend": None,
+            "uses_extend_func": False,
+        }
+
     def test_when_schema(self):
         class Params(Parameters):
             defaults = {
@@ -426,6 +438,22 @@ class TestAccess:
         for param in sorted_tp:
             assert getattr(sorted_tp, param) == getattr(shuffled_tp, param)
 
+    def test_sort_values_no_labels(self):
+        class Params(Parameters):
+            defaults = {
+                "test": {
+                    "title": "test",
+                    "description": "",
+                    "type": "int",
+                    "value": 2,
+                }
+            }
+
+        params = Params()
+        assert params.sort_values() == params._data
+        assert params.sort_values({"test": params.test})
+        assert params.dump()
+
     def test_sort_values_correctness(self):
         """Ensure sort is correct"""
         exp = [
@@ -537,6 +565,26 @@ class TestAccess:
         params.sort_values()
         assert params.extend_param == "don't update me"
 
+    def test_sort_values_with_state(self, extend_ex_path):
+        class ExtParams(Parameters):
+            defaults = extend_ex_path
+            label_to_extend = "d0"
+            array_first = False
+
+        params = ExtParams()
+        params.set_state(d0=[6, 7, 8, 9])
+        params.sort_values()
+        assert params.extend_param == [
+            {"d0": 6, "d1": "c1", "value": 5, "_auto": True},
+            {"d0": 6, "d1": "c2", "value": 6, "_auto": True},
+            {"d0": 7, "d1": "c1", "value": 7},
+            {"d0": 7, "d1": "c2", "value": 8},
+            {"d0": 8, "d1": "c1", "value": 7, "_auto": True},
+            {"d0": 8, "d1": "c2", "value": 8, "_auto": True},
+            {"d0": 9, "d1": "c1", "value": 7, "_auto": True},
+            {"d0": 9, "d1": "c2", "value": 8, "_auto": True},
+        ]
+
 
 class TestAdjust:
     def test_adjust_int_param(self, TestParams):
@@ -1134,6 +1182,22 @@ class TestValidationMessages:
         with pytest.raises(ValidationError):
             params.adjust({"param": params.when_param - 1})
 
+    def test_is_deserialized(self, TestParams):
+        params = TestParams()
+        params._adjust({"min_int_param": [{"value": 1}]}, is_deserialized=True)
+        assert params.min_int_param == [
+            {"label0": "zero", "label1": 1, "value": 1},
+            {"label0": "one", "label1": 2, "value": 1},
+        ]
+
+        params._adjust(
+            {"min_int_param": [{"value": -1}]},
+            raise_errors=False,
+            is_deserialized=True,
+        )
+
+        assert params.errors["min_int_param"] == ["min_int_param -1 < min 0 "]
+
 
 class TestArray:
     def test_to_array(self, TestParams):
@@ -1190,24 +1254,22 @@ class TestArray:
         params.madeup = vi
         params._data["madeup"] = {"value": vi}
         value_items = params.select_eq("madeup", False, **params._state)
-        assert params._resolve_order("madeup", value_items) == (
-            exp_label_order,
-            exp_value_order,
-        )
+        assert params._resolve_order(
+            "madeup", value_items, params.label_grid
+        ) == (exp_label_order, exp_value_order)
 
         # test with specified state.
         exp_value_order = {"label0": ["zero", "one"], "label2": [0, 1]}
         params.set_state(label2=[0, 1])
         value_items = params.select_eq("madeup", False, **params._state)
-        assert params._resolve_order("madeup", value_items) == (
-            exp_label_order,
-            exp_value_order,
-        )
+        assert params._resolve_order(
+            "madeup", value_items, params.label_grid
+        ) == (exp_label_order, exp_value_order)
 
         params.madeup[0]["label1"] = 0
         value_items = params.select_eq("madeup", False, **params._state)
         with pytest.raises(InconsistentLabelsException):
-            params._resolve_order("madeup", value_items)
+            params._resolve_order("madeup", value_items, params.label_grid)
 
     def test_to_array_with_state1(self, TestParams):
         params = TestParams()
@@ -1227,8 +1289,21 @@ class TestArray:
 
         assert res.tolist() == exp
 
-        exp = params.int_dense_array_param
-        assert params.from_array("int_dense_array_param", res) == exp
+        assert (
+            params.from_array("int_dense_array_param", res)
+            == params.int_dense_array_param
+        )
+
+        params = TestParams()
+        res = params.to_array("int_dense_array_param", label0="zero")
+
+        assert res.tolist() == exp
+
+        act = copy.deepcopy(
+            params.from_array("int_dense_array_param", res, label0="zero")
+        )
+        params.set_state(label0="zero")
+        assert act == params.int_dense_array_param
 
     def test_to_array_with_state2(self, TestParams):
         params = TestParams()
@@ -1258,8 +1333,23 @@ class TestArray:
 
         assert res.tolist() == exp
 
-        exp = params.int_dense_array_param
-        assert params.from_array("int_dense_array_param", res) == exp
+        assert (
+            params.from_array("int_dense_array_param", res)
+            == params.int_dense_array_param
+        )
+
+        params = TestParams()
+        res = params.to_array("int_dense_array_param", label1=[0, 1, 2, 5])
+
+        assert res.tolist() == exp
+
+        act = copy.deepcopy(
+            params.from_array(
+                "int_dense_array_param", res, label1=[0, 1, 2, 5]
+            )
+        )
+        params.set_state(label1=[0, 1, 2, 5])
+        assert act == params.int_dense_array_param
 
 
 class TestState:
@@ -1267,32 +1357,32 @@ class TestState:
         params = TestParams()
         assert params.view_state() == {}
         params.set_state(label0="zero")
-        assert params.view_state() == {"label0": "zero"}
+        assert params.view_state() == {"label0": ["zero"]}
         params.set_state(label1=0)
-        assert params.view_state() == {"label0": "zero", "label1": 0}
+        assert params.view_state() == {"label0": ["zero"], "label1": [0]}
         params.set_state(label0="one", label2=1)
         assert params.view_state() == {
-            "label0": "one",
-            "label1": 0,
-            "label2": 1,
+            "label0": ["one"],
+            "label1": [0],
+            "label2": [1],
         }
         params.set_state(**{})
         assert params.view_state() == {
-            "label0": "one",
-            "label1": 0,
-            "label2": 1,
+            "label0": ["one"],
+            "label1": [0],
+            "label2": [1],
         }
         params.set_state()
         assert params.view_state() == {
-            "label0": "one",
-            "label1": 0,
-            "label2": 1,
+            "label0": ["one"],
+            "label1": [0],
+            "label2": [1],
         }
         params.set_state(label1=[1, 2, 3])
         assert params.view_state() == {
-            "label0": "one",
+            "label0": ["one"],
             "label1": [1, 2, 3],
-            "label2": 1,
+            "label2": [1],
         }
 
     def test_label_grid(self, TestParams):
@@ -1567,6 +1657,13 @@ class TestExtend:
         params = AFParams()
         assert isinstance(params.int_dense_array_param, np.ndarray)
         assert params.from_array("int_dense_array_param") == exp
+        for val in params._data["int_dense_array_param"]["value"]:
+            if val["label1"] in (2, 4, 5):
+                assert val["_auto"] is True
+            else:
+                assert "_auto" not in val
+
+        assert params.dump()["int_dense_array_param"]["value"] == exp
 
         class AFParams(Parameters):
             defaults = array_first_defaults
@@ -1598,7 +1695,6 @@ class TestExtend:
 
         params = AFParams()
         assert params.int_dense_array_param.tolist()
-        params.set_state(label0="one")
         exp = [
             {"label0": "one", "label1": 0, "label2": 0, "value": 1},
             {"label0": "one", "label1": 0, "label2": 1, "value": 2},
@@ -1619,7 +1715,13 @@ class TestExtend:
             {"label0": "one", "label1": 5, "label2": 1, "value": 17},
             {"label0": "one", "label1": 5, "label2": 2, "value": 18},
         ]
-        assert params.from_array("int_dense_array_param") == exp
+        assert params.from_array("int_dense_array_param", label0="one") == exp
+
+        for val in params._data["int_dense_array_param"]["value"]:
+            if val["label0"] == "zero":
+                assert val["_auto"] is True
+            else:
+                assert "_auto" not in val
 
     def test_extend_w_array(self, extend_ex_path):
         class ExtParams(Parameters):
@@ -1666,29 +1768,12 @@ class TestExtend:
             [-1, -1],
         ]
 
-        params = ExtParams()
-        params.adjust(
-            {
-                "extend_param": [
-                    {"d0": 3, "d1": "c1", "value": -1},
-                    {"d0": 3, "d1": "c2", "value": 1},
-                ]
-            }
-        )
-
-        assert params.extend_param.tolist() == [
-            [1, 2],
-            [1, 2],
-            [1, 2],
-            [-1, 1],
-            [-1, 1],
-            [-1, 1],
-            [-1, 1],
-            [-1, 1],
-            [-1, 1],
-            [-1, 1],
-            [-1, 1],
-        ]
+        for val in params._data["extend_param"]["value"]:
+            # 0, 1 extended at the beginning.
+            if val["d0"] > 3 or val["d0"] in (0, 1):
+                assert val["_auto"] is True
+            else:
+                assert "_auto" not in val
 
         params = ExtParams()
         params.adjust(
@@ -1767,9 +1852,126 @@ class TestExtend:
 
         params = ExtParams()
         params.adjust({"extend_param": [{"d0": 0, "value": 1}]})
-
         assert params.extend_param.tolist() == [[1, 1]] * 11
 
+    def test_extend_adj_without_clobber(self, extend_ex_path):
+        class ExtParams(Parameters):
+            defaults = extend_ex_path
+            label_to_extend = "d0"
+            array_first = True
+
+        params = ExtParams()
+        params.adjust(
+            {"extend_param": [{"d0": 3, "value": -1}]}, clobber=False
+        )
+        assert params.extend_param.tolist() == [
+            [1, 2],
+            [1, 2],
+            [1, 2],
+            [-1, -1],
+            [-1, -1],
+            [5, 6],
+            [5, 6],
+            [7, 8],
+            [7, 8],
+            [7, 8],
+            [7, 8],
+        ]
+
+        params = ExtParams()
+        params.adjust(
+            {
+                "extend_param": [
+                    {"d0": 3, "d1": "c1", "value": -1},
+                    {"d0": 3, "d1": "c2", "value": 1},
+                ]
+            },
+            clobber=False,
+        )
+
+        assert params.extend_param.tolist() == [
+            [1, 2],
+            [1, 2],
+            [1, 2],
+            [-1, 1],
+            [-1, 1],
+            [5, 6],
+            [5, 6],
+            [7, 8],
+            [7, 8],
+            [7, 8],
+            [7, 8],
+        ]
+
+        params = ExtParams()
+        params.adjust(
+            {
+                "extend_param": [
+                    {"d0": 3, "value": -1},
+                    {"d0": 5, "d1": "c1", "value": 0},
+                    {"d0": 5, "d1": "c2", "value": 1},
+                    {"d0": 8, "d1": "c1", "value": 22},
+                    {"d0": 8, "d1": "c2", "value": 23},
+                ]
+            },
+            clobber=False,
+        )
+
+        assert params.extend_param.tolist() == [
+            [1, 2],
+            [1, 2],
+            [1, 2],
+            [-1, -1],
+            [-1, -1],
+            [0, 1],
+            [0, 1],
+            [7, 8],
+            [22, 23],
+            [22, 23],
+            [22, 23],
+        ]
+
+        params = ExtParams()
+        params.adjust(
+            {
+                "extend_param": [
+                    {"d0": 3, "value": -1},
+                    {"d0": 5, "d1": "c1", "value": 0},
+                    {"d0": 6, "d1": "c2", "value": 1},
+                ]
+            },
+            clobber=False,
+        )
+        assert params.extend_param.tolist() == [
+            [1, 2],
+            [1, 2],
+            [1, 2],
+            [-1, -1],
+            [-1, -1],
+            [0, 6],
+            [0, 1],
+            [7, 8],
+            [7, 8],
+            [7, 8],
+            [7, 8],
+        ]
+
+        params = ExtParams()
+        params.adjust({"extend_param": [{"d0": 0, "value": 1}]}, clobber=False)
+        assert params.extend_param.tolist() == [
+            [1, 1],
+            [1, 1],
+            [1, 2],
+            [3, 4],
+            [3, 4],
+            [5, 6],
+            [5, 6],
+            [7, 8],
+            [7, 8],
+            [7, 8],
+            [7, 8],
+        ]
+
     def test_extend_adj_w_errors(self, extend_ex_path):
         class ExtParams(Parameters):
             defaults = extend_ex_path
@@ -1871,7 +2073,7 @@ class TestExtend:
             [-1, -1],
         ]
 
-    def test_custom_extend(self, extend_ex_path):
+    def test_extend_method(self, extend_ex_path):
         class ExtParams(Parameters):
             defaults = extend_ex_path
             # label_to_extend = "d0"
@@ -1888,18 +2090,63 @@ class TestExtend:
             }
         )
         params.extend(
-            label_to_extend="d0",
-            label_to_extend_values=[2, 4, 7],
-            params="extend_param",
+            label="d0", label_values=[2, 4, 7], params="extend_param"
         )
-
-        assert sorted(params.extend_param, key=lambda vo: vo["d0"]) == [
+        params.sort_values()
+        assert params.extend_param == [
             {"d0": 2, "d1": "c1", "value": 1},
             {"d0": 2, "d1": "c2", "value": 2},
-            {"d0": 4, "d1": "c1", "value": 1},
-            {"d0": 4, "d1": "c2", "value": 2},
-            {"d0": 7, "d1": "c1", "value": 1},
-            {"d0": 7, "d1": "c2", "value": 2},
+            {"d0": 4, "d1": "c1", "value": 1, "_auto": True},
+            {"d0": 4, "d1": "c2", "value": 2, "_auto": True},
+            {"d0": 7, "d1": "c1", "value": 1, "_auto": True},
+            {"d0": 7, "d1": "c2", "value": 2, "_auto": True},
+        ]
+
+        params = ExtParams()
+        init = params.select_eq("extend_param")
+        params.extend(label="d0", label_values=[])
+        assert init == params.select_eq("extend_param")
+
+        params = ExtParams()
+        params.extend(label="d0", label_values=[8, 9, 10])
+        params.sort_values()
+        assert params.extend_param == [
+            {"d0": 2, "d1": "c1", "value": 1},
+            {"d0": 2, "d1": "c2", "value": 2},
+            {"d0": 3, "d1": "c1", "value": 3},
+            {"d0": 3, "d1": "c2", "value": 4},
+            {"d0": 5, "d1": "c1", "value": 5},
+            {"d0": 5, "d1": "c2", "value": 6},
+            {"d0": 7, "d1": "c1", "value": 7},
+            {"d0": 7, "d1": "c2", "value": 8},
+            {"d0": 8, "d1": "c1", "value": 7, "_auto": True},
+            {"d0": 8, "d1": "c2", "value": 8, "_auto": True},
+            {"d0": 9, "d1": "c1", "value": 7, "_auto": True},
+            {"d0": 9, "d1": "c2", "value": 8, "_auto": True},
+            {"d0": 10, "d1": "c1", "value": 7, "_auto": True},
+            {"d0": 10, "d1": "c2", "value": 8, "_auto": True},
+        ]
+
+        params = ExtParams()
+        params.extend(label="d0", label_values=[0, 8, 9, 10])
+        params.sort_values()
+        assert params.extend_param == [
+            {"d0": 0, "d1": "c1", "value": 1, "_auto": True},
+            {"d0": 0, "d1": "c2", "value": 2, "_auto": True},
+            {"d0": 2, "d1": "c1", "value": 1},
+            {"d0": 2, "d1": "c2", "value": 2},
+            {"d0": 3, "d1": "c1", "value": 3},
+            {"d0": 3, "d1": "c2", "value": 4},
+            {"d0": 5, "d1": "c1", "value": 5},
+            {"d0": 5, "d1": "c2", "value": 6},
+            {"d0": 7, "d1": "c1", "value": 7},
+            {"d0": 7, "d1": "c2", "value": 8},
+            {"d0": 8, "d1": "c1", "value": 7, "_auto": True},
+            {"d0": 8, "d1": "c2", "value": 8, "_auto": True},
+            {"d0": 9, "d1": "c1", "value": 7, "_auto": True},
+            {"d0": 9, "d1": "c2", "value": 8, "_auto": True},
+            {"d0": 10, "d1": "c1", "value": 7, "_auto": True},
+            {"d0": 10, "d1": "c2", "value": 8, "_auto": True},
         ]
 
 
@@ -1936,6 +2183,13 @@ class TestIndex:
             [3, 3],
         ]
 
+        for val in params._data["indexed_param"]["value"]:
+            # 0, 1 extended at the beginning.
+            if val["d0"] > 3 or val["d0"] in (0, 1):
+                assert val["_auto"] is True
+            else:
+                assert "_auto" not in val
+
         class IndexParams2(Parameters):
             defaults = extend_ex_path
             label_to_extend = "d0"
