@@ -1,5 +1,6 @@
 import json
 import os
+from bisect import bisect_left, bisect_right
 from collections import OrderedDict
 from typing import List
 
@@ -82,9 +83,9 @@ def consistent_labels(value_items: List[ValueObject]):
     """
     if not value_items:
         return set([])
-    used = set(k for k in value_items[0] if k != "value")
+    used = set(k for k in value_items[0] if k not in ("value", "_auto"))
     for vo in value_items:
-        if used != set(k for k in vo if k != "value"):
+        if used != set(k for k in vo if k not in ("value", "_auto")):
             return None
     return used
 
@@ -102,7 +103,11 @@ def hashable_value_object(vo: ValueObject) -> tuple:
     Helper function convertinga value object into a format
     that can be stored in a set.
     """
-    return tuple(sorted(vo.items()))
+    return tuple(
+        (label, value)
+        for label, value in sorted(vo.items())
+        if label not in ("_auto",)
+    )
 
 
 def filter_labels(vo: ValueObject, drop=None, keep=None) -> ValueObject:
@@ -124,7 +129,11 @@ def make_label_str(vo: ValueObject) -> str:
     Create string from labels. This is used to create error messages.
     """
     lab_str = ", ".join(
-        [f"{lab}={vo[lab]}" for lab in sorted(vo) if lab != "value"]
+        [
+            f"{lab}={vo[lab]}"
+            for lab in sorted(vo)
+            if lab not in ("value", "_auto")
+        ]
     )
     if lab_str:
         return f"[{lab_str}]"
@@ -140,3 +149,36 @@ def grid_sort(vos, label_to_extend, grid):
             return grid[0]
 
     return sorted(vos, key=key)
+
+
+class SortedKeyList:
+    """
+    Sorted key list inspired by SortedContainers and the Python docs:
+
+    - http://www.grantjenks.com/docs/sortedcontainers/
+    - https://docs.python.org/3.9/library/bisect.html
+    """
+
+    def __init__(self, values, keyfunc):
+        self.sorted_key_list = [(val, keyfunc(val)) for val in values]
+        self.sorted_key_list.sort(key=lambda r: r[1])
+        self.keys = [r[1] for r in self.sorted_key_list]
+        self.keyfunc = keyfunc
+
+    def lte(self, value):
+        i = bisect_right(self.keys, self.keyfunc(value))
+        if i:
+            return self.sorted_key_list[i - 1][0]
+        return None
+
+    def gte(self, value):
+        i = bisect_left(self.keys, self.keyfunc(value))
+        if i != len(self.keys):
+            return self.sorted_key_list[i][0]
+        return None
+
+    def insert(self, value):
+        key_value = self.keyfunc(value)
+        i = bisect_left(self.keys, key_value)
+        self.sorted_key_list.insert(i, (value, key_value))
+        self.keys.insert(i, key_value)
